package no.rwr.plaf;

import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.plaf.LabelUI;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Font;
import java.awt.FontMetrics;

public class GLabelUI extends LabelUI {

    public static ComponentUI createUI(JComponent label) {
	return new GLabelUI((JLabel) label);
    }


    public GLabelUI(JLabel label) {
    }



    public Dimension getPreferredSize(JComponent component) {
	JLabel label = (JLabel) component;

	String text = label.getText();
	
	
	// text width and height:
	int tw = 0;
	int th = 0;
	Font font = label.getFont();
	if (font != null) {
	    FontMetrics fm = label.getFontMetrics(font);
	    tw = fm.stringWidth(text);
	    th = fm.getHeight();
	}

	int iw, ih, gap;
	Icon icon = label.getIcon();
	if (icon == null) {
	    iw = ih = gap = 0;
	} else {
	    iw = icon.getIconWidth();
	    ih = icon.getIconHeight();
	    gap = label.getIconTextGap();
	}
	
	int width, height;

	// See comment in JLabel concerning layout.
	if (label.getHorizontalTextPosition() == JLabel.CENTER) {
	    width = Math.max(tw, iw);
	    height = (label.getVerticalTextPosition() == JLabel.CENTER) ?
		Math.max(th, ih) :
		(th + ih + gap);
	} else {
	    width = tw + iw + gap;
	    height = Math.max(th, ih);
	}
	
	Insets insets = label.getInsets();
	width += insets.left + insets.right;
	height += insets.top + insets.bottom;

	return new Dimension(width, height);
    }

    public void paint(Graphics gfx, JComponent component) {

	JLabel label = (JLabel) component;

	String text = label.getText();

	Font font = label.getFont();
	FontMetrics fm = label.getFontMetrics(font);
	
	
	// Text size
	int tw = fm.stringWidth(text);
	int th = fm.getHeight();

	int iw, ih, gap;
	Icon icon = label.getIcon();
	if (icon == null) {
	    iw = ih = gap = 0;
	} else {
	    iw = icon.getIconWidth();
	    ih = icon.getIconHeight();
	    gap = label.getIconTextGap();
	}

	// Size of contents
	int width, height;

	// Text pos within contents
	int tx, ty;

	// Image pos wihin contents
	int ix, iy;

	int hTextPos = label.getHorizontalTextPosition();

	switch (hTextPos) {
	case JLabel.CENTER:
	    width = Math.max(tw, iw);
	    tx = (width - tw)/2;
	    ix = (width - iw)/2;
	    break;

	case JLabel.LEFT:
	case JLabel.LEADING:  // FIXME, don't assume
	    tx = 0;
	    ix = tw + gap;
	    width = ix + iw;
	    break;
	    
	case JLabel.RIGHT:
	case JLabel.TRAILING:  // FIXME, don't assume
	    ix = 0;
	    tx = iw + gap;
	    width = tx + tw;
	    break;
	    
	default:
	    throw new Error("unknown alignment (" + hTextPos + ")");
	}

	switch (label.getVerticalTextPosition()) {

	case JLabel.CENTER:
	    height = Math.max(th, ih);
	    ty = (height - th)/2;
	    iy = (height - ih)/2;
	    break;
	    
	case JLabel.TOP:
	case JLabel.LEADING:  // FIXME, don't assume
	    if (hTextPos == JLabel.CENTER) {
		ty = 0;
		iy = th + gap;
		height = iy + ih;
	    } else {
		height = Math.max(th, ih);
		ty = 0;
		iy = 0;
	    }
	    break;
	    
	case JLabel.BOTTOM:
	case JLabel.TRAILING:  // FIXME, don't assume
	    if (hTextPos == JLabel.CENTER) {
		iy = 0;
		ty = ih + gap;
		height = ty + th;
	    } else {
		height = Math.max(th, ih);
		ty = height - th;
		iy = height - ih;
	    }
	    break;
		
	default:
	    throw new Error("unknown alignment");
	}

	ty += fm.getAscent();

	int xOffset;
	int yOffset;

	int cWidth = label.getWidth();
	int cHeight = label.getHeight();

	Insets insets = label.getInsets();
	cWidth -= insets.left + insets.right;
	cHeight -= insets.top + insets.bottom;
	
	switch (label.getHorizontalAlignment()) {
	case JLabel.CENTER:
	    xOffset = (cWidth - width) / 2;

	    break;

	case JLabel.LEFT:
	case JLabel.LEADING:  // FIXME, don't assume
	    xOffset = 0;
	    break;
	    
	case JLabel.RIGHT:
	case JLabel.TRAILING:  // FIXME, don't assume
	    xOffset = cWidth - width;
 	    break;

	default:
	    throw new Error("unknown alignment");
	}

	switch (label.getVerticalAlignment()) {
	case JLabel.CENTER:
	    yOffset = (cHeight - height) / 2;
	    break;

	case JLabel.TOP:
	case JLabel.LEADING:  // FIXME, don't assume
	    yOffset = 0;
	    break;
	    
	case JLabel.BOTTOM:
	case JLabel.TRAILING:  // FIXME, don't assume
	    yOffset = cHeight - height;
 	    break;

	default:
	    throw new Error("unknown alignment");
	}

	xOffset += insets.left;
	yOffset += insets.top;

	icon.paintIcon(label, gfx, xOffset+ix, yOffset+iy);

	// draw text
	gfx.setColor(label.getForeground());
	gfx.setFont(font);
	gfx.drawString(text, xOffset+tx, yOffset+ty);
    }
}
