
#include <X11/Xlib.h>

#include <gcj/cni.h>
#include <gnu/gcj/RawData.h>
#include <java/lang/String.h>

#include "no/rwr/x/Display.h"
#include "no/rwr/x/Font.h"
#include "no/rwr/x/XException.h"

/*
jint no::rwr::x::Font::loadFontXID(Display* display, jstring lfdNamePattern) {
    ::Display* dpy = (::Display*) display->display;
    int len = JvGetStringUTFLength(lfdNamePattern);
    char cName[len+1];
    JvGetStringUTFRegion(lfdNamePattern, 0, lfdNamePattern->length(),
			 cName);
    cName[len] = '\0';

    //int xid = XLoadFont(dpy, cName);

    XFontStruct fontStruct = XLoadQueryFont(dpy, cName);
    return xid;
}
*/

gnu::gcj::RawData* no::rwr::x::Font::loadFont(Display* display,
					      jstring lfdNamePattern) {
    ::Display* dpy = (::Display*) display->display;
    int len = JvGetStringUTFLength(lfdNamePattern);
    char cName[len+1];
    JvGetStringUTFRegion(lfdNamePattern, 0, lfdNamePattern->length(),
			 cName);
    cName[len] = '\0';

    XFontStruct* fontStruct = XLoadQueryFont(dpy, cName);
    if (fontStruct == 0) {
	JvThrow(new XException(JvNewStringLatin1("font not found")));
    }

    return reinterpret_cast<gnu::gcj::RawData*>(fontStruct);

}

jint no::rwr::x::Font::getXIDFromStruct(gnu::gcj::RawData* structure) {
    XFontStruct* fontStruct = (XFontStruct*) structure;
    return fontStruct->fid;
}

jint no::rwr::x::Font::getMaxAscent() {
    XFontStruct* fontStruct = (XFontStruct*) structure;
    return fontStruct->max_bounds.ascent;
}

jint no::rwr::x::Font::getMaxDescent() {
    XFontStruct* fontStruct = (XFontStruct*) structure;
    return fontStruct->max_bounds.descent;
}

jint no::rwr::x::Font::getAscent() {
    XFontStruct* fontStruct = (XFontStruct*) structure;
    return fontStruct->ascent;
}

jint no::rwr::x::Font::getDescent() {
    XFontStruct* fontStruct = (XFontStruct*) structure;
    return fontStruct->ascent;
}

jint no::rwr::x::Font::getStringWidth(java::lang::String* text) {
    //::Display* dpy = (::Display*) display->display;
    XFontStruct* fontStruct = (XFontStruct*) structure;


    // FIXME: make proper unicode conversion
    int len = JvGetStringUTFLength(text);
    char ctxt[len+1];
    JvGetStringUTFRegion(text, 0, text->length(), ctxt);
    ctxt[len] = '\0';
    int width = XTextWidth(fontStruct, ctxt, len);
}

void no::rwr::x::Font::finalize() {
    if (structure != 0) {
	::Display* dpy = (::Display*) display->display;
	XFontStruct* fontStruct = (XFontStruct*) structure;
	int result = XFreeFont(dpy, fontStruct);
	if (result == BadFont) JvThrow(new XException(display, result));

        structure = 0; xid = 0;
    }
}

