
#include <vector>

#include <X11/Xlib.h>

#include <gcj/cni.h>
#include <gnu/gcj/RawData.h>
#include <java/lang/String.h>
#include <java/awt/Rectangle.h>

#include "no/rwr/x/Display.h"
#include "no/rwr/x/XID.h"
#include "no/rwr/x/Drawable.h"
#include "no/rwr/x/Font.h"
#include "no/rwr/x/XImage.h"
#include "no/rwr/x/XException.h"
#include "no/rwr/x/Clip.h"
#include "no/rwr/x/GC.h"
#include "no/rwr/x/XException.h"

#include <iostream>

typedef java::awt::Rectangle AWTRect;
typedef JArray<AWTRect*> AWTRectArray;
typedef vector<XRectangle> XRectVector;

void no::rwr::x::GC::initStructure(GC* copyFrom) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    
    ::GC gc = XCreateGC(dpy, drawableXID, 0, 0);

    if (gc == 0) JvThrow(new XException(JvNewStringLatin1("GC creation "
							  "failed")));

    if (copyFrom != 0) {
	::GC fromGC = (::GC) copyFrom->structure;
	XCopyGC(dpy, fromGC, ~0, gc);
	// no fast fail
    }

    structure = reinterpret_cast<gnu::gcj::RawData*>(gc);
}

void no::rwr::x::GC::disposeImpl() {
    gnu::gcj::RawData* lStructure = structure;
    Drawable* lTargetType = targetType;

    if ((lStructure == 0) || (lTargetType == 0)) return;
    
    structure = 0;
    targetType = 0;
	
    Display* display = lTargetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::GC gc = (::GC) lStructure;
	
    XFreeGC(dpy, gc);
    // no fast fail
}

void no::rwr::x::GC::setForeground(jlong pixel) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::GC gc = (::GC) structure;
    XSetForeground(dpy, gc, pixel);
    // no fast fail
}

void no::rwr::x::GC::setFont(Font* font) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::GC gc = (::GC) structure;
    XSetFont(dpy, gc, font->getXID());
    // no fast fail
}

void no::rwr::x::GC::drawString(jstring text, jint x, jint y) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;

    /*
    jint length = text->length();
    jchar* txt = JvGetStringChars(text);

    XChar2b xwchars[length];
    
    // FIXME: Add convertion and caching

    for (int i=0; i<length; i++) {
	XChar2b* xc = &(xwchars[i]);
	jchar jc = txt[i];
	xc->byte1 = jc & 0xff;
	xc->byte2 = jc >> 8;
    }

     XDrawString16(dpy, drawableXID, gc, x, y, xwchars, length);
    */

    // ALTERNATIVE:
    int len = JvGetStringUTFLength(text);
    char ctxt[len+1];
    JvGetStringUTFRegion(text, 0, text->length(), ctxt);
    ctxt[len] = '\0';
    XDrawString(dpy, drawableXID, gc, x, y, ctxt, len);
    // no fast fail
}

void no::rwr::x::GC::drawLine(jint x1, jint y1, jint x2, jint y2) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;
    XDrawLine(dpy, drawableXID, gc, x1, y1, x2, y2);
    // no fast fail
}

void no::rwr::x::GC::drawRectangle(jint x, jint y, jint w, jint h) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;
    XDrawRectangle(dpy, drawableXID, gc, x, y, w, h);
    // no fast fail
}

void no::rwr::x::GC::fillRectangle(jint x, jint y, jint w, jint h) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;
    XFillRectangle(dpy, drawableXID, gc, x, y, w, h);
    // no fast fail
}

void no::rwr::x::GC::clearArea(jint x, jint y, jint w, jint h,
			       jboolean exposures) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();

    XClearArea(dpy, drawableXID, x, y, w, h,
	       exposures ? True : False);
    // no fast fail
}


void no::rwr::x::GC::putImage(XImage* image,
			      jint srcX, jint srcY,
			      jint destX, jint destY,
			      jint width, jint height) {
    
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;
    ::XImage* ximage = (::XImage*) (image->structure);

    XPutImage(dpy, drawableXID, gc, ximage,
	      srcX, srcY,
	      destX, destY,
	      width, height);
    // no fast fail
}

void no::rwr::x::GC::updateClip() {
    if (clip == 0) return;

    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) structure;

    XRectVector* xrectvector = (XRectVector*) (clip->xrects);
    int numRect = xrectvector->size();
        
    int originX = 0;
    int originY = 0;
    int ordering = Unsorted;
    XSetClipRectangles(dpy, gc, originX, originY,
		       xrectvector->begin(), numRect,
		       ordering);
    // no fast fail
}
