#include <vector>

#include <X11/Xlib.h>

#include <gcj/cni.h>
#include "java/awt/Rectangle.h"
#include "no/rwr/x/Atom.h"
#include "no/rwr/x/Display.h"
#include "no/rwr/x/Window.h"
#include "no/rwr/x/WindowAttributes.h"
#include "no/rwr/x/Visual.h"
#include "no/rwr/x/XException.h"

jint no::rwr::x::Window::createChildXID(java::awt::Rectangle* bounds,
					jint borderWidth,
					WindowAttributes* attributes,
					jint windowIOClass, 
					Visual* visual) {
    ::Window parentXID = xid;
    
    int x      = bounds->x;
    int y      = bounds->y;
    int width  = bounds->width;
    int height = bounds->height;
    
    long mask = attributes->mask;
    XSetWindowAttributes* attr = (XSetWindowAttributes*)
	attributes->getXSetWindowAttributesStructure();
    
    ::Visual* vis = CopyFromParent;
    int depth = CopyFromParent;
    if (visual != 0) {
	vis = (::Visual*) visual->getVisualStructure();
	depth = visual->getDepth();
    } 

    ::Window childXID = XCreateWindow((::Display*) (display->display),
				      parentXID,
				      x, y, width, height,
				      borderWidth, depth, windowIOClass,
				      vis,
				      mask, attr);
    // no fast fail
    return childXID;
}

void no::rwr::x::Window::destroy() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XDestroyWindow(dpy, window);
    // no fast fail
}

void no::rwr::x::Window::setAttributes(WindowAttributes* attributes) {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    ::XSetWindowAttributes* attr = (::XSetWindowAttributes*)
	  attributes->getXSetWindowAttributesStructure();

    XChangeWindowAttributes(dpy, window, attributes->mask, attr);
    // no fast fail
}






void no::rwr::x::Window::map() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XMapWindow(dpy, window);
    // no fast fail
}

void no::rwr::x::Window::unmap() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XUnmapWindow(dpy, window);
    // no fast fail
}


// FIXME: this doesn't belong here...
void no::rwr::x::Window::withdraw() {
    ::Display* dpy = (::Display*) (display->display);
    ::Window window = xid;
    XUnmapWindow(dpy, window);
    // no fast fail
}


void no::rwr::x::Window::setProperty(jint nameAtom, jint typeAtom, 
				     jbyteArray data) {
    ::Display* dpy = (::Display*) (display->display);
    int format = 8;
    int mode = PropModeReplace;
    unsigned char* pData = (unsigned char*) elements(data);
    int len = data->length;
    
    XChangeProperty(dpy, xid, nameAtom, typeAtom, format, mode,
		    pData, len);
    // no fast fail
}

void no::rwr::x::Window::setWMProtocols(jintArray atoms) {
     ::Display* dpy = (::Display*) (display->display);
     
     size_t length = atoms->length;
     jint* atomsBegin = elements(atoms);
     jint* atomsEnd   = atomsBegin + length;

     // Avoid confusion between Xlib.h and Atom.java "Atom" types.
     typedef ::Atom XLibAtom;

     std::vector<XLibAtom> atomVector(atomsBegin, atomsEnd);
     XLibAtom* atomsArray = atomVector.begin();

     XSetWMProtocols(dpy, xid, atomsArray, length);
     // no fail fast
}

jintArray no::rwr::x::Window::getWMProtocols() {
    ::Display* dpy = (::Display*) (display->display);

    ::Atom* protocolsReturn;
    int countReturn;

    Status success = XGetWMProtocols(dpy, xid, &protocolsReturn,
				     &countReturn);

    if (!success)
	JvThrow(new XException(JvNewStringLatin1("cannot get "
						 "WM protocols ")));

    jintArray atoms;
    try {

	::Atom* protocolsBegin = protocolsReturn;
	::Atom* protocolsEnd = protocolsBegin + countReturn;
	
	atoms = JvNewIntArray(countReturn);
	jint* atomsBegin = elements(atoms);
	
	std::copy(protocolsBegin, protocolsEnd, atomsBegin);
    
    } catch (...) {
	XFree(protocolsReturn);
	throw;
    }
    XFree(protocolsReturn);
    
    return atoms;
}

void no::rwr::x::Window::setBounds(jint x, jint y,
				   jint width, jint height) {
    ::Display* dpy = (::Display*) (display->display);

    XMoveResizeWindow(dpy, xid, x, y, width, height);
    // no fast fail
}
